import subprocess
from lib_pipeline.singleton import Singleton
from lib_pipeline.helpers import execute


class Docker(object):
    __metaclass__ = Singleton

    def pull(self, image):
        return execute(f"docker pull {image}")

    def run(self, image, *commands, cwd=None, env_args="", options=""):
        command = " ".join(arg for arg in commands)
        return execute(
            f"""docker run -it -v $(pwd):/home/src -w /home/src {env_args} {image} {command} {options}""",
            cwd,
        )



def echo(command):
    execute(f"echo {command}")


def execute(command, cwd=None, shell=True):
    try:
        from collections import namedtuple
        import subprocess

        print(command)
        proc = subprocess.run(
            command,
            cwd=cwd,
            shell=shell,
            check=True,
            encoding="utf-8",
            stdout=subprocess.PIPE,
        )
        print(proc.stdout)
        output = namedtuple("output", ["stdout", "stderr", "returncode"])
        return output(proc.stdout, proc.stderr, proc.returncode)
    except subprocess.CalledProcessError:
        raise


def remove_files(directory, cwd):
    return execute(f"rm -rf {directory}", cwd)


def zip(output_filename, dir_name, format="zip"):
    import shutil

    shutil.make_archive(output_filename, format, dir_name)


def unzip(path_to_zip_file, directory_to_extract_to):
    import zipfile

    zip_ref = zipfile.ZipFile(path_to_zip_file, "r")
    zip_ref.extractall(directory_to_extract_to)
    zip_ref.close()



from lib_pipeline.helpers import execute
import json


class AWS:
    def run(self, resource, command, cwd=None):
        return execute(f"aws {resource} {command}", cwd)


class S3(AWS):
    def __init__(self):
        self.resource = "s3"

    def copy(self, bucket_path, files, *args, **kwargs):
        options = " ".join(arg for arg in args)
        cwd = kwargs.get("cwd")
        return self.run(
            self.resource, f"cp {files} s3://{bucket_path} {options}", cwd=cwd
        )

    def empty(self, bucket_path, *args):
        options = " ".join(arg for arg in args)
        return self.run(self.resource, f"rm s3://{bucket_path} {options}")


class Route53(AWS):
    def __init__(self):
        self.resource = "route53"

    def list_resource_record_sets(self, hosted_zone_id, *args):
        options = " ".join(arg for arg in args)
        return self.run(
            self.resource,
            f"list-resource-record-sets --hosted-zone-id {hosted_zone_id} {options}",
        )

    def get_active_region(self, hosted_zone_id, domain, dns_prefix, *args):
        result = self.list_resource_record_sets(
            hosted_zone_id,
            f"--query 'ResourceRecordSet[?starts_with(Name,`{dns_prefix}`)]'",
            *args,
        )
        for i in json.loads(result.stdout)["ResourceRecordSets"]:
            if i["Name"] == domain and i["Failover"] == "PRIMARY":
                return i["SetIdentifier"].split(f"{dns_prefix}-")[1]
        return None



from setuptools import setup, find_packages

setup(
    name="lib_pipeline",
    version="0.0.3",
    author="Blake Lassiter",
    author_email="blakelass@gmail.com",
    description="A python implementation of services to help deploy using aws and terraform",
    url="https://github.com/balassit/lib_pipeline",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    zip_safe=False,
)



import subprocess
from lib_pipeline.docker import Docker
from lib_pipeline.singleton import Singleton
from lib_pipeline.helpers import execute, remove_files


class Terraform(object):
    __metaclass__ = Singleton

    def __init__(self):
        self.docker = Docker()
        self.image = "hashicorp/terraform:light"
        self.docker_env_args = '-e "HOME=/home" -v $HOME/.aws:/home/.aws'

    def exec(self, command, options=None, cwd=None):
        options = " ".join(arg for arg in options)
        self.docker.run(
            self.image, command, options=options, cwd=cwd, env_args=self.docker_env_args
        )

    def init(self, env, region, cwd):
        backend_options = [
            f"""--backend-config="profile={env}" """,
            f"""--backend-config="region={region}" """,
            f"""--backend-config="bucket=sharebuilder401k-terraform-{env}-{region}" """,
        ]
        self.exec("init", options=backend_options, cwd=cwd)

    def plan(self, env, region, options, cwd=None):
        vars = [
            f"""--var-file="{env}/{region}.tfvars" """,
            f"""--var="profile={env}" {options} """,
        ]
        self.exec("plan", options=vars, cwd=cwd)

    def apply(self, env, region, options, cwd=None):
        vars = [
            f"""--var-file="{env}/{region}.tfvars" """,
            f"""--var="profile={env}" {options} """,
            "--auto-approve",
        ]

        self.exec("apply", options=vars, cwd=cwd)

    def deploy(self, profile, region, env, *args, cwd=None):
        options = " ".join(arg for arg in args)
        remove_files(".terraform", cwd=cwd)
        self.init(env, region, cwd=cwd)
        self.plan(env, region, options, cwd=cwd)
        self.apply(env, region, options, cwd=cwd)

    def taint(self, profile, region, env, *args, cwd=None):
        options = " ".join(arg for arg in args)
        self.init(env, region, cwd=cwd)
        self.plan(env, region, options, cwd=cwd)
        self.exec("taint", options=options, cwd=cwd)
